/** \file setup.c
 * \brief   
 *
 * \author ARV
 * \date	19 . 2017 .
 * \copyright Copyright 2015  ARV. All rights reserved.
 */

#include <avr/io.h>
#include "avr_helper.h"
#include <avr/eeprom.h>
#include <stdlib.h>
#include <string.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "power_control.h"
#include "global.h"
#include "lcd.h"
#include "lcd_show.h"
#include "hardware.h"
#include "setup.h"
#include "control.h"
#include "ir_remote.h"
#include "menu.h"

/**
 * \defgroup MENU  
 *   
 *
 * \addtogroup MENU
 * @{
 *
 *     -   MENU
 *     -  
 *    (   ) -  .
 *      -    15 .
 *       :    ,
 *   -  (  ,  ).
 *
 *         , .. 
 *   .
 */

__flash const char VOL_NAME[]		= "SENSITIVITY";
static __flash const char str2[]	= "A.G.C.";
static __flash const char str3[]	= "SLEEP TIME";
static __flash const char str4[]	= "WS2812 IN GROUP";
static __flash const char str5[]	= "GROUPS OF WS2812";
static __flash const char str6[]	= "RESET TO DEFAULT";
static __flash const char str7[]	= "IR REMOTE MENU";
static __flash const char str8[]	= "DC OFFSET";

static __flash const char ir0[]		= "POWER OFF";
static __flash const char ir1[]		= "VOLUME+";
static __flash const char ir2[]		= "VOLUME-";
static __flash const char ir3[]		= "EFFECT NEXT";
static __flash const char ir4[]		= "EFFECT PREV";
static __flash const char ir5[]		= "PRESET NEXT";
static __flash const char ir6[]		= "PRESET PREV";
static __flash const char ir7[]		= "DONE UP LEARNING";

static void default_setup(void){
	cfg.pixels_in_group = 1;
	cfg.group_of_pixels = 96;
	cfg.time_to_sleep = 0;
	cfg.sensitivity = 50;
	cfg.agc_enabled = 0;
}

static void default_ir_setup(void){
#include "ir_car_mp3.h"
	init_ir_cmd_code(0, CMD_VOLUME_PLUS, IR_VOL_PLUS);
	init_ir_cmd_code(1, CMD_VOLUME_MINUS, IR_VOL_MINUS);
	init_ir_cmd_code(2, CMD_EFFECT_PLUS, IR_CH_PLUS);
	init_ir_cmd_code(3, CMD_EFFECT_MINUS, IR_CH_MINUS);
	init_ir_cmd_code(4, CMD_PRESET_PLUS, IR_NEXT);
	init_ir_cmd_code(5, CMD_PRESET_MINUS, IR_PREV);
	init_ir_cmd_code(6, CMD_POWER, IR_NO_CMD);
}

static EEMEM config_t e_cfg = {
	.clear = 0xFF
};

void update_config(void){
	eeprom_update_block(&cfg, &e_cfg, sizeof(config_t));
}

INIT(6){
	//     ,  lcd_show !
	eeprom_read_block(&cfg, &e_cfg, sizeof(config_t));
	if(cfg.clear){
		memset(&cfg, 0, sizeof(config_t));
		default_setup();
		cfg.band_mask[0] = BAND_LF;
		cfg.band_mask[1] = BAND_MF;
		cfg.band_mask[0] = BAND_HF;
		cfg.dc_offset = 0x7FE0;
		eeprom_update_block(&cfg, &e_cfg, sizeof(config_t));
		default_ir_setup();
	}
}

static menu_result_t reset_to_default(uint16_t d){
	default_setup();
	lcd_clrscr();
	lcd_puts_P("ALL SETTINGS\nRESET TO DEFAULT");
	_delay_ms(1000);
	return MENU_DONE;
}

#define DC_OFS_MIN	29696UL
#define DC_OFS_MAX	35200UL

static void edit_dc(int8_t d, uint16_t data){
	uint16_t tmp = cfg.dc_offset;
	tmp += d*0x40;
	if(tmp < DC_OFS_MIN) tmp = DC_OFS_MIN;
	else if(tmp > DC_OFS_MAX) tmp = DC_OFS_MAX;
	cfg.dc_offset = tmp;
}

static void paint_dc(int32_t d){
	char tmp[10];
	signal_t *ptr = (void*)(int)d;

	lcd_gotoxy(0,1);
	lcd_puts_P("DC=");
	itoa(ptr->average_vol, tmp, 10);
	lcd_puts(tmp);
	lcd_puts_P("/");
	utoa(cfg.dc_offset, tmp, 10);
	lcd_puts(tmp);
	lcd_puts_P("         ");
}

#define ir_wait lock_current_menu_item
static uint32_t ir_code;

static void ir_paint(int32_t d){
	static uint8_t div;

	uint32_t tmp;
	tmp = ir_wait ? get_ir_code() : get_ir_cmd_code(d);
	if(tmp) ir_code = tmp;

	lcd_gotoxy(0,1);
	lcd_puts_P("CODE: ");

	if((++div & 0x20) && ir_wait)
		lcd_puts_P("          ");
	else
		if((ir_code == 0) || (ir_code == IR_NO_CMD)){
			lcd_puts_P("--------  ");
		} else {
			show_hex_long(ir_code);
		}

	if(ir_wait) reset_menu_timeout();
}

static menu_result_t ir_action(uint16_t d){
	if(ir_wait){
		set_ir_cmd_code(d, ir_code);
		ir_wait = 0;
	} else {
		ir_wait = 1;
		ir_code = 0;
		set_ir_cmd_code(d, IR_NO_CMD);
	}
	return MENU_CONTINUE;
}

static menu_result_t ir_action_reset(uint16_t d){
	default_ir_setup();
	lcd_clrscr();
	lcd_puts_P("IR-REMOTE RESET\nTO DEFAULT");
	return MENU_DONE_MSG;
}

static void edit_pix(int8_t d, uint16_t data){
	// data -     cfg
	int16_t tmp = *(uint8_t *)data;
	uint8_t gop = cfg.group_of_pixels;
	uint8_t pig = cfg.pixels_in_group;

	if((uint8_t *)data == &cfg.group_of_pixels){
		// - 
		tmp += d*STEP_GROUP_OF_PIX;
		if(tmp < MIN_GROUP_OF_PIX) tmp = MIN_GROUP_OF_PIX;
		if(tmp > MAX_GROUP_OF_PIX) tmp = MAX_GROUP_OF_PIX;
	} else {
		//   
		tmp += d*STEP_PIX_IN_GROUP;
		if(tmp < MIN_PIX_IN_GROUP) tmp = MIN_PIX_IN_GROUP;
		if(tmp > MAX_PIX_IN_GROUP) tmp = MAX_PIX_IN_GROUP;
	}
	*(uint8_t *)data = tmp;
	if((cfg.group_of_pixels * cfg.pixels_in_group) > MAX_TOTAL_PIX){
		cfg.group_of_pixels = gop;
		cfg.pixels_in_group = pig;
	}
}

static void paint_pix(int32_t d){
	show_number(*(uint8_t*)(uint16_t)d);
}

static menu_result_t update_and_reboot(uint16_t d){
	update_config();
	lcd_clrscr();
	lcd_puts_P("ALL SETTINGS\nSAVED");
	_delay_ms(1000);
	return MENU_DONE;
}

static __flash const menu_item_t __flash const ir_menu_items[] = {
	_MI_USER(ir0, NULL, ir_paint, ir_action, CMD_POWER),
	_MI_USER(ir1, NULL, ir_paint, ir_action, CMD_VOLUME_PLUS),
	_MI_USER(ir2, NULL, ir_paint, ir_action, CMD_VOLUME_MINUS),
	_MI_USER(ir3, NULL, ir_paint, ir_action, CMD_EFFECT_PLUS),
	_MI_USER(ir4, NULL, ir_paint, ir_action, CMD_EFFECT_MINUS),
	_MI_USER(ir5, NULL, ir_paint, ir_action, CMD_PRESET_PLUS),
	_MI_USER(ir6, NULL, ir_paint, ir_action, CMD_PRESET_MINUS),
	_MI_USER(ir7, NULL, show_press_encoder_msg, NULL, 0),
	_MI_USER(str6, NULL, show_press_encoder_msg, ir_action_reset, 0)
};

static __flash const menu_t ir_menu = _MENU(ir_menu_items);

static __flash const menu_item_t __flash const main_menu_items[] = {
	_MI_SCALE(VOL_NAME, cfg.sensitivity, 0, MAX_LEVEL, 1),
	_MI_U8(str3, cfg.time_to_sleep, 0, 60, 5),
	_MI_ONOFF(str2, cfg.agc_enabled),
	_MI_USER(str5, edit_pix, paint_pix, update_and_reboot, &cfg.group_of_pixels),
	_MI_USER(str4, edit_pix, paint_pix, update_and_reboot, &cfg.pixels_in_group),
	_MI_USER(str8, edit_dc, paint_dc, update_and_reboot, &music),
	_MI_SUBMENU(str7, ir_menu),
	_MI_USER(str6, NULL, show_press_encoder_msg, reset_to_default,0)
};

__flash const menu_t main_menu = _MENU(main_menu_items);

/**
 * @}
 */
